package tests;
import static org.junit.Assert.*;

import org.junit.*;

import expr.*; 

public class TestChartData {
	
	ChartData cd = new ChartData() ;
	
	
	@After public void checkInvariant( ) {
		// It's not always the case that a class invariant can be
		// checked using only the public interface. Better to 
		// put checks into the mutators and constructors, as
		// suggested in the assignment. That also ensures that the
		// checking happens during system checking as well as
		// unit checking.
		assertNotNull( cd.getExpression() ) ;
		assertTrue( Double.NEGATIVE_INFINITY < cd.getXMin( ) ) ;
		assertTrue( cd.getXMin( ) < cd.getXMax( ) ) ;
		assertTrue( cd.getXMax( ) < Double.POSITIVE_INFINITY ) ;
		assertTrue( Double.NEGATIVE_INFINITY < cd.getYMin( ) ) ;
		assertTrue( cd.getYMin( ) < cd.getYMax( ) ) ;
		assertTrue( cd.getYMax( ) < Double.POSITIVE_INFINITY ) ;
	}

	@Test public void testBasics() {
		
		checkInvariant( ) ;
		
		cd.setXRange(-20.5, +20.5);
		assertEquals( -20.5, cd.getXMin(), 0.00001 );
		assertEquals( 20.5, cd.getXMax(), 0.00001 );
		
		cd.setYRange(-100.001, -11.001);
		assertEquals( -100.001, cd.getYMin(), 0.00001 );
		assertEquals( -11.001, cd.getYMax(), 0.00001 );
		Expression a = (new ExpressionFactory()).x() ;
		cd.setExpression( a );
		assertEquals(a, cd.getExpression() ); 	
		checkInvariant( ) ;
	}
	
	@Test public void preconditionChecking0() {
		try {
			cd.setXRange( 12, 12 );
			assertTrue( "Precondition violated, but no exception thrown.", false) ; }
		catch( AssertionError ex) { }
		catch( Throwable ex) {
			assertTrue( "Precondition violated, but exception thrown was not an AssertionError.", false) ; }
	}

	@Test public void preconditionChecking1() {
	try {
		cd.setXRange( Double.NEGATIVE_INFINITY, 12 );
		assertTrue( "Precondition violated, but no exception thrown.", false) ; }
	catch( AssertionError ex) { }
	catch( Throwable ex) {
		assertTrue( "Precondition violated, but exception thrown was not an AssertionError.", false) ; }
	}

	@Test public void preconditionChecking2() {
	try {
		cd.setXRange( 12, Double.POSITIVE_INFINITY );
		assertTrue( "Precondition violated, but no exception thrown.", false) ; }
	catch( AssertionError ex) { }
	catch( Throwable ex) {
		assertTrue( "Precondition violated, but exception thrown was not an AssertionError.", false) ; }
	}

	@Test public void preconditionChecking3() {
	try {
		cd.setXRange( 12, Double.NaN );
		assertTrue( "Precondition violated, but no exception thrown.", false) ; }
	catch( AssertionError ex) { }
	catch( Throwable ex) {
		assertTrue( "Precondition violated, but exception thrown was not an AssertionError.", false) ; }
	}
	
	@Test public void preconditionChecking4() {
		try {
			cd.setYRange( 12, 12 );
			assertTrue( "Precondition violated, but no exception thrown.", false) ; }
		catch( AssertionError ex) { }
		catch( Throwable ex) {
			assertTrue( "Precondition violated, but exception thrown was not an AssertionError.", false) ; }
	}

	@Test public void preconditionChecking5() {
	try {
		cd.setYRange( Double.NEGATIVE_INFINITY, 12 );
		assertTrue( "Precondition violated, but no exception thrown.", false) ; }
	catch( AssertionError ex) { }
	catch( Throwable ex) {
		assertTrue( "Precondition violated, but exception thrown was not an AssertionError.", false) ; }
	}

	@Test public void preconditionChecking6() {
	try {
		cd.setYRange( 12, Double.POSITIVE_INFINITY );
		assertTrue( "Precondition violated, but no exception thrown.", false) ; }
	catch( AssertionError ex) { }
	catch( Throwable ex) {
		assertTrue( "Precondition violated, but exception thrown was not an AssertionError.", false) ; }
	}

	@Test public void preconditionChecking7() {
	try {
		cd.setYRange( 12, Double.NaN );
		assertTrue( "Precondition violated, but no exception thrown.", false) ; }
	catch( AssertionError ex) { }
	catch( Throwable ex) {
		assertTrue( "Precondition violated, but exception thrown was not an AssertionError.", false) ; }
	}

	@Test public void preconditionChecking8() {
	try {
		cd.setExpression( null ) ;
		assertTrue( "Precondition violated, but no exception thrown.", false) ; }
	catch( AssertionError ex) { }
	catch( Throwable ex) {
		assertTrue( "Precondition violated, but exception thrown was not an AssertionError.", false) ; }
	}
}
